/*
  Author:  Evan Carmichael, modified hydroponics control for "vertical flow" setup.
  Idea and logicloop based on Tom De Bie "Billie"'s hydroponics controller. Easy IoT and 
  watchdog timer based on Lostangles hydroponic controller.
  Copyright (C) 2017  Evan Carmichael

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.

  Originally based on Billie's hydroponics controller:
  https://github.com/BillieBricks/Billie-s-Hydroponic-Controller
  and
  Lostangles hydroponics controller:
  https://github.com/lostangles/hydroponics

  
  
  Changes include:
  -pump control for an vertical flow system
  -modified pH controller for black pH probe from china
  -EasyIoT incorportation to report status of each sensor over the web
  -removed unused features, refer to the original github above


*/

#include <dht11.h>               //DHT Temp and humidity
#include <Esp8266EasyIoTMsg.h>
#include <Esp8266EasyIoTConfig.h>
#include <Esp8266EasyIoT.h>
#include <SPI.h>
#include <DS1302.h>
#include <Wire.h>                //One Wire library
#include <EEPROMex.h>            //Extended Eeprom library
#include <avr/wdt.h>




#define dht_dpin 49                //pin for DHT11
int pHPin = A7;                    //pin for pH probe
int pHPlusPin = 45;                //pin for Acid pump (relay)
int pHMinPin = 43;                 //pin for Base pump (relay)
int pumpPin = 47;				           //pin for main pump
int floatLowPin = 11;              //pin for lower float sensor
int floatHighPin = 12;             //pin for upper float sensor
int solenoidPin = 26;              //pin for Tank fill Solenoid valve (relay)


// Variables
int x, y;                          //xy axis for touch
int page = 0;                      //returns what page your on
int tankProgState = 1;             //returns the state of tank program - on or off
int manualRefilState = 0;
float pH;                          //generates the value of pH
int phUp = 2;						           //Always let IoT flip state on startup
int phDown = 2;					           //             |

float Setpoint;                    //holds value for Setpoint
float HysterisMin;                 //Minimum deviation from Setpoint
float HysterisPlus;                //Maximum deviation from Setpoint
float SetHysteris;                 //Holds value for Hysteris
float FanTemp;
float FanHumid;

int pinTime = LOW;								   // Variables for pulsing the pump
unsigned long previousMillis = 0;             //             |
long pinHighTime = 1000;              //             |
long pinLowTime = 7500;              //             |


int EepromSetpoint = 10;      //location of Setpoint in Eeprom
int EepromSetHysteris = 20;   //location of SetHysteris in Eeprom
int EepromPinHighTime = 80;   //location of pinHighTime in Eeprom
int EepromPinLowTime = 100;   //location of pinLowTime in Eeprom
int EepromSmoothPh = 90;      //location of smoothPh in Eeprom

Esp8266EasyIoT esp;

dht11 dht;
#define DHT11PIN 49;
float lastTemp;
float lastHum;
int lastpH;
int lastFlood;
int lastpHPluspin;
int lastpHMinpin;
boolean phChange = true;

#define CHILD_ID_HUM 0
#define CHILD_ID_TEMP 1
#define CHILD_ID_PH 3
#define CHILD_ID_FLOOD 4
#define CHILD_ID_PH_UP 5
#define CHILD_ID_PH_DOWN 6
#define CHILD_ID_WTEMP 7
Esp8266EasyIoTMsg msgHum(CHILD_ID_HUM, V_HUM);
Esp8266EasyIoTMsg msgTemp(CHILD_ID_TEMP, V_TEMP);
Esp8266EasyIoTMsg msgpH(CHILD_ID_PH, V_PH);
Esp8266EasyIoTMsg msgFlood(CHILD_ID_FLOOD, V_PH);
Esp8266EasyIoTMsg msgPHup(CHILD_ID_PH_UP, V_DOOR);
Esp8266EasyIoTMsg msgPHdown(CHILD_ID_PH_DOWN, V_DOOR);
Esp8266EasyIoTMsg msgWaterTemp(CHILD_ID_WTEMP, V_TEMP);


void setup()
{
  MCUSR = 0;
  wdt_disable();
  wdt_reset();	                                   //Watch dog threshold reset
  noScreenSetup();                                 //
  EepromRead();                                    // Reads every value that is stored in Eeprom
  logicSetup();                                    // Replaces the void Setup

}

void loop()
{
  wdt_reset();	     //Watch dog threshold reset
  logicLoop();       //pH algorithm loops through this one, also the smoothing of the signal
  wdt_reset();	     //Watch dog threshold reset
  TankProgControl(); //Controls loop for refilling the tank
  wdt_reset();	     //Watch dog threshold reset
  IoTreport();       //ESP8266 WiFi report for EasyIoT
  wdt_reset();	    //Watch dog threshold reset
}

void EepromRead()   //Reads Eeprom value for the following variables
{
  Setpoint = EEPROM.readFloat(EepromSetpoint);
  SetHysteris = EEPROM.readFloat(EepromSetHysteris);
  pinHighTime = EEPROM.readLong(EepromPinHighTime);
  pinLowTime = EEPROM.readLong(EepromPinLowTime);

}



void logicSetup()
{

  pinMode(pHPlusPin, OUTPUT);
  pinMode(pHMinPin, OUTPUT);
  pinMode(pumpPin, OUTPUT);
  pinMode (solenoidPin, OUTPUT);

  Serial.begin(9600);
  delay(300);
  Serial.println("Setting up output pins...");
  delay(700);
  Wire.begin();
}


void logicLoop()
{
  float sensorValue = 0;                               // Default the Value = 0
  sensorValue = analogRead(pHPin);                     // sensor value gets the value from pH probe
  pH = phTest();

  HysterisMin = (Setpoint - SetHysteris);              // lowest sensor devation further that this and BASE is added
  HysterisPlus = (Setpoint + SetHysteris);             // highest sensor devation further than this and ACID is added

  if (pH == Setpoint)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;

      digitalWrite(pHMinPin, LOW);                    // set BASE pump to off position
      digitalWrite(pHPlusPin, LOW);                   // set ACID pump to off position
      Serial.println("ph + and - are LOW");
      phDown = 0;
      phUp = 0;
      pinTime =  pinLowTime;
    }
  }

  if (pH >= HysterisMin && pH <= HysterisPlus )
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;
      digitalWrite(pHMinPin, LOW);
      digitalWrite(pHPlusPin, LOW);
      Serial.println("ph + and - are HIGH (hysteria correction)");
      phUp = 0;
      phDown = 0;
      pinTime = pinLowTime;
    }
  }

  if (pH < HysterisMin)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;
      digitalWrite(pHPlusPin, LOW);
      digitalWrite(pHMinPin, HIGH);
      Serial.println("ph + pin is HIGH");
      phUp = 0;
      phDown = 1;
      pinTime = pinHighTime;
    }
  }

  if (pH >= HysterisMin && pH < Setpoint)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;
      digitalWrite(pHPlusPin, LOW);
      digitalWrite(pHMinPin, HIGH);
      Serial.println("ph - pin is HIGH");
      phUp = 0;
      phDown = 1;
      pinTime = pinHighTime;
    }
  }

  if (pH > HysterisPlus)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;
      digitalWrite(pHMinPin, HIGH);
      digitalWrite(pHPlusPin, LOW);
      Serial.println("ph + pin is HIGH");
      phDown = 0;
      phUp = 1;
      pinTime = pinLowTime;
    }
  }

  if (pH <= HysterisPlus && pH > Setpoint)
  {
    unsigned long currentMillis = millis();
    if (currentMillis - previousMillis > pinTime)
    {
      previousMillis = currentMillis;
      digitalWrite(pHMinPin, HIGH);
      digitalWrite(pHPlusPin, LOW);
      Serial.println("ph - pin is HIGH");
      phUp = 0;
      phDown = 1;
      pinTime = pinLowTime;
    }
  }
  Serial.print("Setpoint = ");
  Serial.println(Setpoint);
  Serial.print("Hysteris = ");
  Serial.println(SetHysteris);
  Serial.print("pH = ");
  Serial.println(pH);


  delay(500);
}



void TankProgControl()
{
  if (tankProgState == 0)
  {
  }
  if (tankProgState == 1)
  {
    int levelHigh = LOW;
    int levelLow = LOW;

    levelHigh = digitalRead(floatHighPin);
    levelLow = digitalRead(floatLowPin);

    if (levelHigh == LOW)
    {
      if (page == 0)

        if (levelLow == LOW)
        {
          if (page == 0)
          {

          }
          digitalWrite(solenoidPin, HIGH); //solenoid valve open.
        }

    }
    else
    {
      if (page == 0)
      {

      }
      if (levelLow == HIGH)
      {
        digitalWrite(solenoidPin, LOW); //solenoid valve closed.
        if (page == 3)
        {

        }
      }
    }
  }
  delay(500);
}







void IoTsetup() {
  Serial2.begin(9600);
  Serial2.setTimeout(1000);
  Serial.println("EasyIoTEsp init");

  dht.read(49);

  pinMode(13, OUTPUT);

  esp.begin(NULL, 3, &Serial2, &Serial);
  esp.present(CHILD_ID_HUM, S_HUM);
  esp.present(CHILD_ID_TEMP, S_TEMP);
  esp.present(CHILD_ID_PH, V_PH);
  esp.present(CHILD_ID_FLOOD, V_PH);
  esp.present(CHILD_ID_PH_UP, S_DOOR);
  esp.present(CHILD_ID_PH_DOWN, S_DOOR);
  esp.present(CHILD_ID_WTEMP, S_TEMP);

}

void IoTreport() {
  Serial.println("Test");
  while (!esp.process());

  delay(100);

  while (!esp.process());

  dht.read(49);
  float temperature = dht.temperature;
  if (isnan(temperature)) {
    Serial.println("Failed reading temperature from DHT");
  }


  float humidity = dht.humidity;
  if (isnan(humidity)) {
    Serial.println("Failed reading humidity from DHT");
  }
  else if (humidity != lastHum)
  {
    lastHum = humidity;
    esp.send(msgHum.set(humidity, 1));
    Serial.print("H: ");
    Serial.println(humidity);
  }
  int pHlvl = phTest() * 10;

  if ((pHlvl - lastpH) > 1 || (lastpH - pHlvl) > 1)
  {
    lastpH = pHlvl;
    esp.send(msgpH.set(phTest(), 2));
    Serial.print("pH: ");
    Serial.println(phTest());
    phChange = true;
  }


  if (phChange)
  {
    lastpHPluspin = phUp;
    esp.send(msgPHup.set(phUp));
    phChange = false;
  }
  if (phChange)
  {
    lastpHMinpin = phDown;
    esp.send(msgPHdown.set(phDown));
    phChange = false;
  }


}

float phTest() {
  int measure = analogRead(pHPin);
  Serial.print("Measure: ");
  Serial.print(measure);

  double voltage = 5 / 1024.0 * measure; //classic digital to voltage conversion
  Serial.print("\tVoltage: ");
  Serial.print(voltage, 3);

  // PH_step = (voltage@PH7 - voltage@PH4) / (PH7 - PH4)
  // PH_probe = PH7 - ((voltage@PH7 - voltage@probe) / PH_step)
  float pH = 11 + ((2.5 - voltage) / 0.18);
  Serial.print("\tPH: ");
  Serial.print(pH, 3);
  return pH;            // pH = the calculated value
  Serial.println("");
  delay(2000);
                      
 
}



void noScreenSetup() {
  EEPROM.writeFloat(EepromSetpoint, 6);  // 6.0 pH for universal plant growth
  EEPROM.writeFloat(EepromSetHysteris, 1.0);
}
